#!/usr/bin/perl
use strict;
use warnings;

my $queryName = "querySeq111";

my ($ref_names1, $ref_recs1) = &read_TrimalOut("010outfile.html");
my ($temp,  $ref_recs2) = &trimalOutToSharp($ref_names1, $ref_recs1);
my ($temp2, $querySeq1) = &queryRecPicker($queryName, $ref_recs2);
my ($ref_recsShortRates, $ref_recsExtraRates) = &rateCalculator($querySeq1, $ref_recs2);
my %recsShortRates = %$ref_recsShortRates;
my %recsExtraRates = %$ref_recsExtraRates;

print("## ShortRates\n");
foreach my $name (keys %recsShortRates){
    print "$name:\t$recsShortRates{$name}\n";
}
print("## LongRates\n");
foreach my $name (keys %recsExtraRates){
    print "$name:\t$recsExtraRates{$name}\n";
}
exit;
###

sub read_TrimalOut{
    my ($InfileNameFN) = @_;
    open(IN, "$InfileNameFN") or die "Cannod find $InfileNameFN\n";
    my @lines = <IN>;
    close(IN);

    my @namesFN = ();
    my %recsFN = ();
    foreach my $Line (@lines){
        #print "$Line";
        chomp($Line);
        if ($Line =~ /    <span class=sel>/){
            my ($name, $seq) = split(/<\/span> {2,}/, $Line);
            #print "name|$name|\n";
            #print "seq|$seq|\n";
            #exit;
            $name =~ s/ +<span class=sel>//;
            $name =~ s/ +$//;
            #print("name: $name\n");
            my @hitNames = grep(/$name/, @namesFN);
            if (scalar(@hitNames) < 1){
                push(@namesFN, $name);
                $recsFN{$name} = "";
            }
            $recsFN{$name} .= $seq;
        #print "$name\n";
        #print "$recsFN{$name}\n";
        #exit;
        }
    }
    close IN;
    #foreach my $name (@namesFN){
    #    print "$name\n";
    #    print "$recsFN{$name}\n";
    #}
    #exit();
    return (\@namesFN, \%recsFN);
}

sub queryRecPicker{
    my ($queryNameFN, $ref_recsFN) = @_;
    my %recsFN = %$ref_recsFN;
    my $keyWord = "^".$queryNameFN."\$";
    #print "keyWord|$keyWord|\n";
    my @hitNames = grep(/$keyWord/, keys %recsFN);
    unless (scalar(@hitNames) == 1){
        print("Stopped. Zero or > 2 hitNames: @hitNames\n");
        print("check queryName:, $queryNameFN\n");
        exit;
    }
    my $queryName1FN = $hitNames[0];
    my $querySeq1FN  = $recsFN{$queryName1FN};
    return ($queryName1FN, $querySeq1FN);
}


sub trimalOutToSharp{
    my ($ref_namesFN, $ref_recsFN) = @_;
    my @namesFN = @$ref_namesFN;
    my %recsFN  = %$ref_recsFN;
    
    my @names2FN = ();
    my %recs2FN  = ();
    foreach my $nameFN (@namesFN){
        #print "$nameFN\n";
        my $seqFN = $recsFN{$nameFN};
        $seqFN =~ s|<span class=sel>-</span>|-|g;
        $seqFN =~ s|<span class=sel>X</span>|X|g;
        $seqFN =~ s|<span class=sel>(\w)</span>|#|g;
        #print("seqFN|$seqFN|\n");
        push(@names2FN, $nameFN);
        $recs2FN{$nameFN} = $seqFN;
    }
    return (\@names2FN, \%recs2FN);
}


sub compare2seqsUse{
    my ($querySeq, $otherSeq) = @_;
    my $countQuerySeq = 0;
    my $countOtherSeq = 0;
    for (my $i=0; $i < length($querySeq); $i++){
        my $queryChr = substr($querySeq, $i, 1);
        my $otherChr = substr($otherSeq, $i, 1);
        #print "$queryChr\n";
        #print "$otherChr\n";
        if ($queryChr eq "#"){
            $countQuerySeq++;
            if ($otherChr eq "#"){
                $countOtherSeq++;
            }
        }
        #print "$countQuerySeq\n";
        #print "$countOtherSeq\n";
        #print "\n";
     }   
    my $round = sprintf("%.3f", $countOtherSeq/$countQuerySeq);
    return ($round);
}

sub compare2seqsExtra{
    my ($querySeq, $otherSeq) = @_;
    my $countQuerySeq = 0;
    my $countOtherSeq = 0;
    for (my $i=0; $i < length($querySeq); $i++){
        my $queryChr = substr($querySeq, $i, 1);
        my $otherChr = substr($otherSeq, $i, 1);
        if ($queryChr eq "#"){
            $countQuerySeq++;
        }
        else{
            if (($otherChr =~ /\w/) and ($otherChr ne "X")){
                $countOtherSeq++;
            }
            if ($otherChr eq "#"){
                $countOtherSeq++;
            }
        }
        #print(countQuerySeq)
        #print(countOtherSeq)
        #print()
    }
    my $round = sprintf("%.3f", $countOtherSeq/$countQuerySeq);
    return ($round);
}

sub rateCalculator{
    my ($querySeq, $ref_recsFN) = @_;
    my %recsFN = %$ref_recsFN;
    my %recsUseRateFN = ();
    my %recsExtraRateFN = ();
    for my $name (keys %recsFN){
        #print "name: $name\n";
        my $sequence = $recsFN{$name};
        #print "recsFN{name}: $recsFN{$name}\n";

        my $useRate = &compare2seqsUse($querySeq, $sequence);
        $recsUseRateFN{$name} = $useRate;

        my $extraRate = &compare2seqsExtra($querySeq, $sequence);
        $recsExtraRateFN{$name} = $extraRate;
    }

    return (\%recsUseRateFN, \%recsExtraRateFN);
}